@@ -1,6 +1,11 @@
 """
-Stack images
+Functions for stacking images (of potentially different sizes) together in a
+single image.
+
+Notes:
+    * We may change the "bg_value" argument to "bg_color" in the future.
 """
+
 import cv2
 import numpy as np
 import skimage
@@ -22,8 +27,15 @@ def stack_images(images, axis=0, resize=None, interpolation=None, overlap=0,
 
         resize (int | str | None):
             if None image sizes are not modified, otherwise resize resize can
-            be either 0 or 1.  We resize the `resize`-th image to match the `1
-            - resize`-th image. Can also be strings "larger" or "smaller".
+            be either 0 or 1.  We resize the `resize`-th image to match the
+            `1 - resize`-th image. In other words, resize=0 means the current
+            image is resized to match the next image, and resize=1 means the
+            next image is resized to match the current image.
+            Can also be strings "larger" or "smaller". If resize="larger", the
+            current image is only resized if the next image is larger. If
+            resize='smaller' the current image is resized only if the next
+            image is smaller.
+            TODO: this needs to be reworked to be more intuitive.
 
         interpolation (int | str):
             string or cv2-style interpolation type.
@@ -41,8 +53,8 @@ def stack_images(images, axis=0, resize=None, interpolation=None, overlap=0,
             if True, returns transforms (scales and translations) to map from
             original image to its new location.
 
-        bg_value (Number | ndarray):
-            background value, if specified, uses this as a fill value.
+        bg_value (Number | ndarray | str):
+            background value or color, if specified, uses this as a fill value.
 
         allow_casting (bool):
             if True, then if "uint255" and "float01" format images are given
@@ -58,15 +70,22 @@ def stack_images(images, axis=0, resize=None, interpolation=None, overlap=0,
     SeeAlso:
         :func:`kwimage.im_stack.stack_images_grid`
 
+    TODO:
+        - [ ] This is currently implemented by calling the "stack_two_images"
+              function multiple times. This should be optimized by allocating
+              the entire canvas first.
+
     Example:
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> import kwimage
         >>> img1 = kwimage.grab_test_image('carl', space='rgb')
         >>> img2 = kwimage.grab_test_image('astro', space='rgb')
         >>> images = [img1, img2]
-        >>> imgB, transforms = stack_images(images, axis=0, resize='larger',
-        >>>                                 overlap=-10, return_info=True)
+        >>> imgB, transforms = kwimage.stack_images(
+        >>>     images, axis=0, resize='larger', pad=10, return_info=True)
         >>> print('imgB.shape = {}'.format(imgB.shape))
         >>> # xdoctest: +REQUIRES(--show)
+        >>> # xdoctest: +REQUIRES(module:kwplot)
         >>> import kwplot
         >>> import kwimage
         >>> kwplot.autompl()
@@ -80,10 +99,12 @@ def stack_images(images, axis=0, resize=None, interpolation=None, overlap=0,
         >>> kwplot.draw_boxes(kwimage.Boxes([xywh1], 'xywh'), color=(1.0, 0, 0))
         >>> kwplot.draw_boxes(kwimage.Boxes([xywh2], 'xywh'), color=(1.0, 0, 0))
         >>> kwplot.show_if_requested()
-        ((662, 512, 3), (0.0, 0.0), (0, 150))
     """
     imgiter = iter(images)
-    img1 = next(imgiter)
+    try:
+        img1 = next(imgiter)
+    except StopIteration:
+        raise ValueError('need at least one image to stack')
 
     if pad is not None:
         overlap = -pad
@@ -120,7 +141,8 @@ def stack_images(images, axis=0, resize=None, interpolation=None, overlap=0,
 
 
 def stack_images_grid(images, chunksize=None, axis=0, overlap=0, pad=None,
-                      return_info=False, bg_value=None, allow_casting=True):
+                      return_info=False, bg_value=None, resize=None,
+                      allow_casting=True):
     """
     Stacks images in a grid. Optionally return transforms of original image
     positions in the output image.
@@ -146,8 +168,12 @@ def stack_images_grid(images, chunksize=None, axis=0, overlap=0, pad=None,
         return_info (bool): if True, returns transforms (scales and
             translations) to map from original image to its new location.
 
-        bg_value (Number | ndarray) : background value, if specified,
-            uses this as a fill value.
+        resize (int | str | None):
+            if None image sizes are not modified, otherwise can be set to
+            "larger" or "smaller" to resize the images in each stack direction.
+
+        bg_value (Number | ndarray | str):
+            background value or color, if specified, uses this as a fill value.
 
         allow_casting (bool):
             if True, then if "uint255" and "float01" format images are given
@@ -162,21 +188,48 @@ def stack_images_grid(images, chunksize=None, axis=0, overlap=0, pad=None,
 
     SeeAlso:
         :func:`kwimage.im_stack.stack_images`
+
+    Example:
+        >>> # xdoctest: +REQUIRES(module:cv2)
+        >>> import kwimage
+        >>> img1 = kwimage.grab_test_image('carl')
+        >>> img2 = kwimage.grab_test_image('astro')
+        >>> img3 = kwimage.grab_test_image('airport')
+        >>> img4 = kwimage.grab_test_image('paraview')[..., 0:3]
+        >>> img5 = kwimage.grab_test_image('pm5644')
+        >>> images = [img1, img2, img3, img4, img5]
+        >>> canvas, transforms = kwimage.stack_images_grid(
+        ...     images, chunksize=3, axis=0, pad=10, bg_value='kitware_blue',
+        ...     return_info=True, resize='larger')
+        >>> print('canvas.shape = {}'.format(canvas.shape))
+        >>> # xdoctest: +REQUIRES(--show)
+        >>> import kwplot
+        >>> import kwimage
+        >>> kwplot.autompl()
+        >>> kwplot.imshow(canvas)
+        >>> kwplot.show_if_requested()
     """
     import ubelt as ub
     if chunksize is None:
         chunksize = int(len(images) ** .5)
     if pad is not None:
         overlap = -pad
+
+    if chunksize == 0:
+        raise ValueError('need at least one image to stack and chunksize cannot be zero')
+
     stack1_list = []
     tfs1_list = []
     assert axis in [0, 1]
-    for batch in ub.chunks(images, chunksize, bordermode='none'):
-        stack1, tfs1 = stack_images(batch, overlap=overlap, return_info=True,
-                                    bg_value=bg_value,
-                                    resize=None, axis=1 - axis)
-        tfs1_list.append(tfs1)
-        stack1_list.append(stack1)
+    try:
+        for batch in ub.chunks(images, chunksize, bordermode='none'):
+            stack1, tfs1 = stack_images(batch, overlap=overlap, return_info=True,
+                                        bg_value=bg_value,
+                                        resize=resize, axis=1 - axis)
+            tfs1_list.append(tfs1)
+            stack1_list.append(stack1)
+    except ZeroDivisionError:
+        raise ValueError('need at least one image to stack')
 
     img_grid, tfs2 = stack_images(stack1_list, overlap=overlap,
                                   bg_value=bg_value,
@@ -331,7 +384,11 @@ def _stack_two_images(img1, img2, axis=0, resize=None, interpolation=None,
         newshape = (hB, wB)
     # Allocate new image for both
     imgB = np.zeros(newshape, dtype=img1.dtype)
+
     if bg_value is not None:
+        if isinstance(bg_value, str):
+            import kwimage
+            bg_value = kwimage.Color(bg_value).forimage(imgB)
         try:
             imgB[:, :] = bg_value
         except ValueError:
