@@ -5,7 +5,7 @@ metadata on top of coordinate data.
 """
 import numpy as np
 import ubelt as ub
-import skimage
+import warnings
 import kwarray
 from kwimage.structs import _generic
 
@@ -14,19 +14,28 @@ try:
 except ImportError:
     from distutils.version import LooseVersion
 
-try:
-    from xdev import profile
-except Exception:
-    from ubelt import identity as profile
-
 
 try:
     import imgaug
     _HAS_IMGAUG_FLIP_BUG = LooseVersion(imgaug.__version__) <= LooseVersion('0.2.9') and not hasattr(imgaug.augmenters.size, '_crop_and_pad_kpsoi')
     _HAS_IMGAUG_XY_ARRAY = LooseVersion(imgaug.__version__) >= LooseVersion('0.2.9')
 except ImportError:
+    imgaug = None
     _HAS_IMGAUG_FLIP_BUG = None
     _HAS_IMGAUG_XY_ARRAY = None
+except Exception as ex:
+    imgaug = None
+    warnings.warn(
+        f'The imgaug module was installed, but failed to import. '
+        f'The module is deprecated and is no longer compatible with numpy 2.x. '
+        f'The error was: {ex}')
+    _HAS_IMGAUG_FLIP_BUG = None
+    _HAS_IMGAUG_XY_ARRAY = None
+
+
+__docstubs__ = """
+from kwimage._typing import SKImageGeometricTransform
+"""
 
 
 class Coords(_generic.Spatial, ub.NiceRepr):
@@ -105,6 +114,9 @@ class Coords(_generic.Spatial, ub.NiceRepr):
         self.data = data
         self.meta = meta
 
+    def __array__(self):
+        return np.asarray(self.data)
+
     def __nice__(self):
         data_repr = repr(self.data)
         if '\n' in data_repr:
@@ -144,9 +156,17 @@ class Coords(_generic.Spatial, ub.NiceRepr):
         return self
 
     def is_numpy(self):
+        """
+        Returns:
+            bool
+        """
         return self._impl.is_numpy
 
     def is_tensor(self):
+        """
+        Returns:
+            bool
+        """
         return self._impl.is_tensor
 
     def compress(self, flags, axis=0, inplace=False):
@@ -167,8 +187,8 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             Coords: filtered coords
 
         Example:
-            >>> from kwimage.structs.coords import *  # NOQA
-            >>> self = Coords.random(10, rng=0)
+            >>> import kwimage
+            >>> self = kwimage.Coords.random(10, rng=0)
             >>> self.compress([True] * len(self))
             >>> self.compress([False] * len(self))
             <Coords(data=array([], shape=(0, 2), dtype=float64))>
@@ -196,11 +216,12 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             Coords: filtered coords
 
         Example:
-            >>> self = Coords(np.array([[25, 30, 15, 10]]))
+            >>> import kwimage
+            >>> self = kwimage.Coords(np.array([[25, 30, 15, 10]]))
             >>> self.take([0])
             <Coords(data=array([[25, 30, 15, 10]]))>
             >>> self.take([])
-            <Coords(data=array([], shape=(0, 4), dtype=int64))>
+            <Coords(data=array([], shape=(0, 4), dtype=...))>
         """
         new = self if inplace else self.__class__(self.data, self.meta)
         new.data = self._impl.take(new.data, indices, axis=axis)
@@ -221,20 +242,24 @@ class Coords(_generic.Spatial, ub.NiceRepr):
         new.data = self._impl.astype(new.data, dtype, copy=not inplace)
         return new
 
-    def round(self, inplace=False):
+    def round(self, decimals=0, inplace=False):
         """
-        Rounds data to the nearest integer
+        Rounds data to the specified decimal place
 
         Args:
             inplace (bool): if True, modifies this object
+            decimals (int): number of decimal places to round to
+
+        Returns:
+            Coords: modified coordinates
 
         Example:
             >>> import kwimage
             >>> self = kwimage.Coords.random(3).scale(10)
             >>> self.round()
         """
-        new = self if inplace else self.__class__(self.data, self.meta)
-        new.data = self._impl.round(new.data)
+        new = self if inplace else self.copy()
+        new.data = self._impl.round(new.data, decimals=decimals)
         return new
 
     def view(self, *shape):
@@ -437,14 +462,13 @@ class Coords(_generic.Spatial, ub.NiceRepr):
                     new.data += 10
         return new
 
-    # @profile
     def warp(self, transform, input_dims=None, output_dims=None,
              inplace=False):
         """
         Generalized coordinate transform.
 
         Args:
-            transform (GeometricTransform | ArrayLike | Augmenter | Callable):
+            transform (SKImageGeometricTransform | ArrayLike | Augmenter | Callable):
                 scikit-image tranform, a 3x3 transformation matrix,
                 an imgaug Augmenter, or generic callable which transforms
                 an NxD ndarray.
@@ -470,6 +494,7 @@ class Coords(_generic.Spatial, ub.NiceRepr):
 
         Example:
             >>> from kwimage.structs.coords import *  # NOQA
+            >>> import skimage
             >>> self = Coords.random(10, rng=0)
             >>> transform = skimage.transform.AffineTransform(scale=(2, 2))
             >>> new = self.warp(transform)
@@ -508,26 +533,21 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             >>> assert np.all(self.warp(func).data == 0)
         """
         import kwimage
+        from kwimage._typing import SKImageGeometricTransform
         impl = self._impl
         new = self if inplace else self.__class__(impl.copy(self.data), self.meta)
         if transform is None:
             return new
-        elif isinstance(transform, _generic.ARRAY_TYPES):
+        elif _generic.isinstance_arraytypes(transform):
             matrix = transform
         elif isinstance(transform, kwimage.Linear):
             matrix = np.asarray(transform)
-        elif isinstance(transform, skimage.transform._geometric.GeometricTransform):
+        elif isinstance(transform, SKImageGeometricTransform):
             matrix = transform.params
         else:
 
             ### Try to accept imgaug tranforms ###
-            try:
-                import imgaug
-            except ImportError:
-                pass
-                # import warnings
-                # warnings.warn('imgaug is not installed')
-            else:
+            if imgaug is not None:
                 if isinstance(transform, imgaug.augmenters.Augmenter):
                     return new._warp_imgaug(transform, input_dims, inplace=True)
 
@@ -588,7 +608,7 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             >>> assert np.allclose(self.data[:, 1], new.data[:, 1])
             >>> assert np.allclose(input_dims[0] - self.data[:, 0], new.data[:, 0])
 
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
             >>> kwplot.autompl()
             >>> kwplot.figure(fnum=1, doclf=True)
@@ -616,7 +636,7 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             >>> assert np.allclose(diff, 0, atol=1e-6, rtol=1e-4)
             >>> #assert np.allclose(self.data[:, 1], new.data[:, 1])
             >>> #assert np.allclose(input_dims[0] - self.data[:, 0], new.data[:, 0])
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwimage
             >>> im = kwimage.imresize(kwimage.grab_test_image(), dsize=input_dims[::-1])
             >>> new_im = augmenter.augment_image(im)
@@ -643,7 +663,6 @@ class Coords(_generic.Spatial, ub.NiceRepr):
         new.data = xy
         return new
 
-    # @profile
     def to_imgaug(self, input_dims):
         """
         Translate to an imgaug object
@@ -653,11 +672,12 @@ class Coords(_generic.Spatial, ub.NiceRepr):
 
         Example:
             >>> # xdoctest: +REQUIRES(module:imgaug)
-            >>> from kwimage.structs.coords import *  # NOQA
-            >>> self = Coords.random(10)
+            >>> import kwimage
+            >>> import numpy as np
+            >>> self = kwimage.Coords.random(10)
             >>> input_dims = (10, 10)
             >>> kpoi = self.to_imgaug(input_dims)
-            >>> new = Coords.from_imgaug(kpoi)
+            >>> new = kwimage.Coords.from_imgaug(kpoi)
             >>> assert np.allclose(new.data, self.data)
         """
         import imgaug
@@ -680,6 +700,60 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             kpoi = imgaug.KeypointsOnImage(kps, shape=input_dims)
         return kpoi
 
+    def to_wkt(self):
+        """
+        Convert coordinates to well known text.
+
+        Returns:
+            str
+
+        Example:
+            >>> import kwimage
+            >>> self = kwimage.Coords.random(3, rng=0)
+            >>> self.to_wkt()
+        """
+        return self.to_shapely().wkt
+
+    def to_shapely(self):
+        """
+        Convert coordinates to shapely.
+
+        Returns:
+            shapely.geometry.multipoint.MultiPoint:
+
+        Example:
+            >>> import kwimage
+            >>> self = kwimage.Coords.random(3)
+            >>> geom = self.to_shapely()
+            >>> print(f'geom={geom}')
+        """
+        import shapely
+        geom = shapely.geometry.multipoint.MultiPoint(self.data)
+        return geom
+
+    @classmethod
+    def from_shapely(Coords, geom):
+        """
+        Create a Coords object from shapely.
+
+        Args:
+            geom (shapely.geometry.multipoint.MultiPoint):
+                a shapely multipoint
+
+        Returns:
+            Self:
+
+        Example:
+            >>> import kwimage
+            >>> self = kwimage.Coords.random(3)
+            >>> geom = self.to_shapely()
+            >>> new = kwimage.Coords.from_shapely(geom)
+            >>> assert np.isclose(self.data, new.data).all()
+        """
+        xy = np.array([point.xy for point in geom.geoms])[:, :, 0]
+        self = Coords(xy)
+        return self
+
     @classmethod
     def from_imgaug(cls, kpoi):
         if _HAS_IMGAUG_XY_ARRAY:
@@ -689,7 +763,6 @@ class Coords(_generic.Spatial, ub.NiceRepr):
         self = cls(xy)
         return self
 
-    @profile
     def scale(self, factor, about=None, output_dims=None, inplace=False):
         """
         Scale coordinates by a factor
@@ -749,7 +822,6 @@ class Coords(_generic.Spatial, ub.NiceRepr):
         new.data = data
         return new
 
-    @profile
     def translate(self, offset, output_dims=None, inplace=False):
         """
         Shift the coordinates
@@ -790,7 +862,6 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             data += offset_
         return new
 
-    @profile
     def rotate(self, theta, about=None, output_dims=None, inplace=False):
         """
         Rotate the coordinates about a point.
@@ -837,8 +908,8 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             >>> # Rotate about the center of mass
             >>> about = self.data.mean(axis=0)
             >>> new4 = self.rotate(theta, about=about)
-            >>> # xdoc: +REQUIRES(--show)
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> import kwplot
             >>> kwplot.figure(fnum=1, doclf=True)
             >>> plt = kwplot.autoplt()
@@ -992,8 +1063,8 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             >>> radius = 3.0
             >>> image2 = np.zeros((s, s))
             >>> self.soft_fill(image2, coord_axes=coord_axes, radius=radius)
-            >>> # xdoc: +REQUIRES(--show)
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> import kwplot
             >>> kwplot.autompl()
             >>> kwplot.imshow(image1, pnum=(1, 2, 1))
@@ -1107,7 +1178,7 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             ndarray: image with coordinates drawn on it
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> from kwimage.structs.coords import *  # NOQA
             >>> s = 256
             >>> self = Coords.random(10, meta={'shape': (s, s)}).scale(s)
@@ -1119,8 +1190,8 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             >>> # image = self.draw_on(image, fill_value, coord_axes=[0, 1], interp='nearest')
             >>> # image = self.draw_on(image, fill_value, coord_axes=[1, 0], interp='bilinear')
             >>> # image = self.draw_on(image, fill_value, coord_axes=[1, 0], interp='nearest')
-            >>> # xdoc: +REQUIRES(--show)
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> import kwplot
             >>> kwplot.autompl()
             >>> kwplot.figure(fnum=1, doclf=True)
@@ -1157,14 +1228,16 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             List[mpl.collections.PatchCollection]: drawn matplotlib objects
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> from kwimage.structs.coords import *  # NOQA
             >>> self = Coords.random(10)
-            >>> # xdoc: +REQUIRES(--show)
-            >>> self.draw(radius=3.0, setlim=True)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
-            >>> kwplot.autompl()
-            >>> self.draw(radius=3.0)
+            >>> plt = kwplot.autoplt()
+            >>> self.draw(radius=0.05, alpha=0.8)
+            >>> plt.gca().set_xlim(0, 1)
+            >>> plt.gca().set_ylim(0, 1)
+            >>> plt.gca().set_aspect('equal')
         """
         import matplotlib as mpl
         import kwimage
@@ -1202,8 +1275,8 @@ class Coords(_generic.Spatial, ub.NiceRepr):
             ax.add_collection(col)
 
         if setlim:
-            x1, y1 = self.data.min(axis=0)
-            x2, y2 = self.data.max(axis=0)
+            x1, y1 = self.data.min(axis=0) - radius
+            x2, y2 = self.data.max(axis=0) + radius
 
             if setlim == 'grow':
                 # only allow growth
