@@ -1,5 +1,7 @@
 """
 TODO:
+    - [ ] Rename Heatmap to Raster, then maybe QuantizedRaster, NormalizedRaster, etc...
+
     - [ ] Remove doctest dependency on ndsampler?
 
     - [ ] Remove the datakeys that tries to define what heatmap should represent
@@ -51,7 +53,7 @@ Example:
     >>> bg_idxs = sampler.catgraph.index('background')
     >>> fcn_target = _dets_to_fcmaps(dets, bg_size, input_dims, bg_idxs)
     >>> fcn_target.keys()
-    >>> print('fcn_target: ' + ub.repr2(ub.map_vals(lambda x: x.shape, fcn_target), nl=1))
+    >>> print('fcn_target: ' + ub.urepr(ub.map_vals(lambda x: x.shape, fcn_target), nl=1))
     >>> # xdoctest: +REQUIRES(--show)
     >>> import kwplot
     >>> kwplot.autompl()
@@ -78,18 +80,16 @@ Example:
     >>> kwplot.imshow(raster)
     >>> kwplot.show_if_requested()
 """
-import cv2
 import numpy as np
 import ubelt as ub
-import skimage
 import kwarray
 import functools
 from . import _generic
 
-try:
-    import torch
-except Exception:
-    torch = None
+
+__docstubs__ = """
+from kwimage._typing import SKImageGeometricTransform
+"""
 
 
 class _HeatmapDrawMixin(object):
@@ -99,19 +99,41 @@ class _HeatmapDrawMixin(object):
 
     def _colorize_class_idx(self):
         """
-        """
-        # Ignore cases where index is negative?
-        cidxs = kwarray.ArrayAPI.numpy(self.data['class_idx']).astype(int)
+        Visualization logic for discrete class index rasters.
+        Negative indexes are interpreted as an "ignore" class.
 
+        Example:
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> import kwimage
+            >>> classes = ['A', 'B', 'C']
+            >>> C = len(classes)
+            >>> class_idxs = np.random.randint(0, C, size=(10, 10))
+            >>> class_idxs[0:3] = -100
+            >>> self = kwimage.Heatmap(class_idx=class_idxs, classes=classes)
+            >>> overlay = self.colorize('class_idx')
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> import kwplot
+            >>> kwplot.autompl()
+            >>> kwplot.imshow(overlay)
+        """
         import networkx as nx
         import kwimage
-
+        # Ignore cases where index is negative?
+        cidxs = kwarray.ArrayAPI.numpy(self.data['class_idx']).astype(int, copy=True)
         classes = self.meta['classes']
-        backup_colors = iter(kwimage.Color.distinct(len(classes)))
-
+        if classes is None:
+            # hack to get something, even though we dont have full info as to
+            # what the classes are.
+            import kwcoco
+            kwcoco.CategoryTree
+            num_classes = int(max(cidxs.max(), 0) + 1)
+            classes = kwcoco.CategoryTree.coerce(num_classes)
+        else:
+            num_classes = len(classes)
+        backup_colors = iter(kwimage.Color.distinct(num_classes))
         name_to_color = {}
-
         if hasattr(classes, 'graph'):
+            # If classes has graph metadata with colors, use that.
             name_to_color = nx.get_node_attributes(classes.graph, 'color')
             for node in classes.graph.nodes:
                 color = classes.graph.nodes[node].get('color', None)
@@ -121,6 +143,23 @@ class _HeatmapDrawMixin(object):
         else:
             name_to_color = ub.dzip(classes, backup_colors)
 
+        # Check for special "ignore" indexes
+        index_is_negative = cidxs < 0
+        if index_is_negative.any():
+            # hack to force a new visualizable class that encompases any
+            # negative indexes.
+            # TODO:
+            # - [ ] some mechanism for controlling what color this will be.
+            # - [ ] Should we differentiate between different negative indexes?
+            special_ignore_classname = '__ignore__'
+            classes = list(classes) + [special_ignore_classname]
+            cidxs[index_is_negative] = (len(classes) - 1)
+            if special_ignore_classname not in name_to_color:
+                # hack to use gray for the special ignore color, we should let
+                # the user have more control here.
+                special_ignore_color = kwimage.Color.coerce('kitware_gray').as01()
+                name_to_color[special_ignore_classname] = special_ignore_color
+
         cx_to_color = np.array([name_to_color[cname] for cname in classes])
         colorized = cx_to_color[cidxs]
         return colorized
@@ -259,6 +298,7 @@ class _HeatmapDrawMixin(object):
             colormask = self._colorize_class_idx()
             colormask = kwimage.ensure_alpha_channel(colormask, with_alpha)
             if imgspace:
+                import torch
                 chw = torch.Tensor(colormask.transpose(2, 0, 1))
                 colormask = self._warp_imgspace(chw, interpolation=interpolation).transpose(1, 2, 0)
             return colormask
@@ -349,6 +389,7 @@ class _HeatmapDrawMixin(object):
         """
         import kwimage
         import matplotlib as mpl
+        import cv2
 
         mat = None
         if image is not None:
@@ -356,7 +397,11 @@ class _HeatmapDrawMixin(object):
             if tf is not None:
                 mat = np.linalg.inv(tf.params)
 
-        cmap = mpl.cm.get_cmap('magma')
+        cmap_name = 'magma'
+        try:
+            cmap = mpl.colormaps[cmap_name]
+        except Exception:
+            cmap = mpl.cm.get_cmap(cmap_name)
 
         level_dsize = self.class_probs.shape[-2:][::-1]
 
@@ -437,8 +482,8 @@ class _HeatmapDrawMixin(object):
             - [ ] Find a way to visualize offset, diameter, and class_probs
                   either individually or all at the same time
 
-          CommandLine:
-              xdoctest -m /home/joncrall/code/kwimage/kwimage/structs/heatmap.py
+        CommandLine:
+            xdoctest -m /home/joncrall/code/kwimage/kwimage/structs/heatmap.py
 
         Example:
             >>> # xdoctest: +REQUIRES(module:kwplot)
@@ -559,12 +604,14 @@ class _HeatmapDrawMixin(object):
                     dx, dy, stride=4, scale=1.0, alpha=with_alpha * vec_alpha,
                     color=color)
                 vec_alpha = max(.1, vec_alpha - .1)
+                import torch
                 chw = torch.Tensor(vecmask.transpose(2, 0, 1))
                 vecalign = self._warp_imgspace(chw, interpolation=interpolation)
                 vecalign = vecalign.transpose(1, 2, 0)
                 layers.append(vecalign)
 
         if kpts is not None:
+            import torch
             # TODO: make a nicer keypoint offset vector visuliazation
             if self.data.get('keypoints', None) is not None:
                 keypoints = self.data['keypoints']
@@ -572,7 +619,14 @@ class _HeatmapDrawMixin(object):
                     # color = (np.array(vec_colors[k]) * 255).astype(np.uint8)
                     color = vec_colors[i + E]
 
-                    dy, dx = kwarray.ArrayAPI.numpy(keypoints[:, k])
+                    try:
+                        dy, dx = kwarray.ArrayAPI.numpy(keypoints[:, k])
+                    except Exception:
+                        print('!!!!!!!!')
+                        print(f'keypoints = {ub.urepr(keypoints, nl=1)}')
+                        print('!!!!!!!!')
+                        raise
+
                     vecmask = kwimage.make_vector_field(dx, dy, stride=8,
                                                         scale=0.5,
                                                         alpha=with_alpha *
@@ -610,17 +664,20 @@ class _HeatmapWarpMixin(object):
 
         Example:
             >>> # xdoctest: +REQUIRES(module:torch)
+            >>> # xdoctest: +REQUIRES(module:cv2)
+            >>> from kwimage.structs.heatmap import *  # NOQA
             >>> self = Heatmap.random((120, 130), img_dims=(200, 210), classes=2, nblips=10, rng=0)
             >>> other = Heatmap.random((60, 70), img_dims=(200, 210), classes=2, nblips=10, rng=1)
             >>> other2 = self._align_other(other)
             >>> assert self.shape != other.shape
             >>> assert self.shape == other2.shape
             >>> # xdoctest: +REQUIRES(--show)
+            >>> import kwplot
             >>> kwplot.autompl()
-            >>> kwplot.imshow(self.colorize(0, imgspace=False), fnum=1, pnum=(3, 2, 1))
-            >>> kwplot.imshow(self.colorize(1, imgspace=False), fnum=1, pnum=(3, 2, 2))
-            >>> kwplot.imshow(other.colorize(0, imgspace=False), fnum=1, pnum=(3, 2, 3))
-            >>> kwplot.imshow(other.colorize(1, imgspace=False), fnum=1, pnum=(3, 2, 4))
+            >>> kwplot.imshow(self.colorize(0, imgspace=False), fnum=1, pnum=(2, 2, 1))
+            >>> kwplot.imshow(self.colorize(1, imgspace=False), fnum=1, pnum=(2, 2, 2))
+            >>> kwplot.imshow(other.colorize(0, imgspace=False), fnum=1, pnum=(2, 2, 3))
+            >>> kwplot.imshow(other.colorize(1, imgspace=False), fnum=1, pnum=(2, 2, 4))
         """
         if self is other:
             return other
@@ -646,6 +703,7 @@ class _HeatmapWarpMixin(object):
         DEPRICATE
         """
         import kwimage
+        import cv2
         M = self.tf_data_to_img.params[0:3]
         dsize = tuple(map(int, self.img_dims[::-1]))
         flags = kwimage.im_cv2._coerce_interpolation(interpolation)
@@ -658,6 +716,7 @@ class _HeatmapWarpMixin(object):
         if self.tf_data_to_img is None and self.img_dims is None:
             aligned = chw.cpu().numpy()
         else:
+            import torch
             if self.tf_data_to_img is None:
                 # If img dims are the same then we dont need a transform we
                 # know its identity
@@ -685,10 +744,12 @@ class _HeatmapWarpMixin(object):
 
         Example:
             >>> # xdoctest: +REQUIRES(module:torch)
+            >>> # xdoctest: +REQUIRES(module:cv2)
             >>> self = Heatmap.random(rng=0, dims=(32, 32))
             >>> colormask = self.upscale()
 
         """
+        import torch
         if channel is None:
             chw = torch.Tensor(self.class_probs)
         else:
@@ -696,7 +757,6 @@ class _HeatmapWarpMixin(object):
         aligned = self._warp_imgspace(chw, interpolation=interpolation)
         return aligned
 
-    # @profile
     def warp(self, mat=None, input_dims=None, output_dims=None,
              interpolation='linear', modify_spatial_coords=True,
              int_interpolation='nearest', mat_is_xy=True, version=None):
@@ -734,6 +794,7 @@ class _HeatmapWarpMixin(object):
 
         Example:
             >>> # xdoctest: +REQUIRES(module:torch)
+            >>> # xdoctest: +REQUIRES(module:cv2)
             >>> from kwimage.structs.heatmap import *  # NOQA
             >>> self = Heatmap.random(rng=0, keypoints=True)
             >>> S = 3.0
@@ -748,8 +809,8 @@ class _HeatmapWarpMixin(object):
             >>> assert f2 == S
 
         Example:
-            >>> import kwimage
             >>> # xdoctest: +REQUIRES(module:ndsampler)
+            >>> import kwimage
             >>> self = kwimage.Heatmap.random(dims=(100, 100), dets='coco', keypoints=True)
             >>> image = np.zeros(self.img_dims)
             >>> # xdoctest: +REQUIRES(module:kwplot)
@@ -761,6 +822,7 @@ class _HeatmapWarpMixin(object):
             >>> kwplot.imshow(toshow)
         """
         import kwimage
+        import skimage
 
         if mat is None:
             mat = self.tf_data_to_img.params
@@ -801,6 +863,7 @@ class _HeatmapWarpMixin(object):
         tf = skimage.transform.AffineTransform(matrix=mat_np)
         # hack: need to get a version of the matrix without any translation
         tf_notrans = _remove_translation(tf)
+        import torch
         mat_notrans = torch.Tensor(tf_notrans.params)
 
         if output_dims is None:
@@ -919,6 +982,7 @@ class _HeatmapAlgoMixin(object):
             Heatmap: the combined heatmap
 
         Example:
+            >>> # xdoctest: +REQUIRES(module:cv2)
             >>> # xdoctest: +REQUIRES(module:torch)
             >>> from kwimage.structs.heatmap import *  # NOQA
             >>> a = Heatmap.random((120, 130), img_dims=(200, 210), classes=2, nblips=10, rng=0)
@@ -1053,12 +1117,13 @@ class _HeatmapAlgoMixin(object):
         Example:
             >>> # xdoctest: +REQUIRES(module:ndsampler)
             >>> from kwimage.structs.heatmap import *  # NOQA
+            >>> import torch
             >>> import ndsampler
             >>> catgraph = ndsampler.CategoryTree.demo()
             >>> class_energy = torch.rand(len(catgraph), 32, 32)
             >>> class_probs = catgraph.hierarchical_softmax(class_energy, dim=0)
             >>> self = Heatmap.random(rng=0, dims=(32, 32), classes=catgraph, keypoints=True)
-            >>> print(ub.repr2(ub.map_vals(lambda x: x.shape, self.data), nl=1))
+            >>> print(ub.urepr(ub.map_vals(lambda x: x.shape, self.data), nl=1))
             >>> self.data['class_probs'] = class_probs.numpy()
             >>> channel = catgraph.index('background')
             >>> dets = self.detect(channel, invert=True)
@@ -1153,7 +1218,7 @@ class Heatmap(_generic.Spatial, _HeatmapDrawMixin,
             img_dims (Tuple[H, W] | Tuple[D, H, W]):
                 original image dimension
 
-            tf_data_to_image (skimage.transform._geometric.GeometricTransform):
+            tf_data_to_image (SKImageGeometricTransform):
                 transformation matrix (typically similarity or affine) that
                 projects the given, heatmap onto the image dimensions such that
                 the image and heatmap are spatially aligned.
@@ -1175,8 +1240,9 @@ class Heatmap(_generic.Spatial, _HeatmapDrawMixin,
     Example:
         >>> # xdoctest: +REQUIRES(module:torch)
         >>> from kwimage.structs.heatmap import *  # NOQA
+        >>> import skimage
         >>> import kwimage
-        >>> class_probs = kwimage.grab_test_image(dsize=(32, 32), space='gray')[None, ] / 255.0
+        >>> class_probs = kwimage.grab_test_image(dsize=(32, 32), space='gray')[None, ..., 0] / 255.0
         >>> img_dims = (220, 220)
         >>> tf_data_to_img = skimage.transform.AffineTransform(translation=(-18, -18), scale=(8, 8))
         >>> self = Heatmap(class_probs=class_probs, img_dims=img_dims,
@@ -1189,6 +1255,7 @@ class Heatmap(_generic.Spatial, _HeatmapDrawMixin,
         >>> kwplot.show_if_requested()
 
     Example:
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> # xdoctest: +REQUIRES(module:torch)
         >>> import kwimage
         >>> self = Heatmap.random()
@@ -1225,6 +1292,7 @@ class Heatmap(_generic.Spatial, _HeatmapDrawMixin,
             tf_data_to_img = meta.get('tf_data_to_img', None)
             if tf_data_to_img is not None:
                 if isinstance(tf_data_to_img, np.ndarray):
+                    import skimage
                     meta['tf_data_to_img'] = skimage.transform.AffineTransform(
                         matrix=tf_data_to_img)
 
@@ -1327,6 +1395,7 @@ class Heatmap(_generic.Spatial, _HeatmapDrawMixin,
             Heatmap
 
         Example:
+            >>> # xdoctest: +REQUIRES(module:cv2)
             >>> from kwimage.structs.heatmap import *  # NOQA
             >>> self = Heatmap.random((128, 128), img_dims=(200, 200),
             >>>     classes=3, nblips=10, rng=0, noise=0.1)
@@ -1345,6 +1414,7 @@ class Heatmap(_generic.Spatial, _HeatmapDrawMixin,
             globals().update(xdev.get_func_kwargs(Heatmap.random))
 
         Example:
+            >>> # xdoctest: +REQUIRES(module:cv2)
             >>> # xdoctest: +REQUIRES(module:ndsampler)
             >>> import kwimage
             >>> self = kwimage.Heatmap.random(dims=(50, 200), dets='coco',
@@ -1386,6 +1456,7 @@ class Heatmap(_generic.Spatial, _HeatmapDrawMixin,
             classes = ['class_{}'.format(c) for c in range(classes)]
 
         # Pretend this heatmap corresponds to some upscaled subregion
+        import skimage
         if img_dims is None:
             scale = 1 + rng.rand(2) * 2
             translation = rng.rand(2) * np.array(dims[::-1]) / 2
@@ -1561,7 +1632,9 @@ def _prob_to_dets(probs, diameter=None, offset=None, class_probs=None,
             detections.
 
     Example:
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> rng = np.random.RandomState(0)
         >>> probs = rng.rand(3, 3).astype(np.float32)
         >>> min_score = .5
@@ -1574,6 +1647,7 @@ def _prob_to_dets(probs, diameter=None, offset=None, class_probs=None,
         >>> assert len(dets) == 9
 
     Example:
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> # xdoctest: +REQUIRES(module:torch)
         >>> import kwimage
         >>> from kwimage.structs.heatmap import *
@@ -1605,6 +1679,7 @@ def _prob_to_dets(probs, diameter=None, offset=None, class_probs=None,
         dets.draw(setlim=True, radius=.1)
 
     Example:
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> heatmap = Heatmap.random(rng=0, dims=(3, 3), diameter=1)
         >>> probs = heatmap.class_probs[0]
         >>> diameter = heatmap.diameter
@@ -1744,19 +1819,23 @@ def smooth_prob(prob, k=3, inplace=False, eps=1e-9):
 
         sigma=0.8 @ k=3, sigma=1.1 @ k=5, sigma=1.4 @ k=7
     """
-    sigma = 0.3 * ((k - 1) * 0.5 - 1) + 0.8  # opencv formula
-    blur = cv2.GaussianBlur(prob, (k, k), sigma)
-    # Shift and scale the intensities so the maximum and minimum
-    # pixel value in the blurred image match the original image
-    minpos = np.unravel_index(blur.argmin(), blur.shape)
-    maxpos = np.unravel_index(blur.argmax(), blur.shape)
-    shift = prob[minpos] - blur[minpos]
-    scale = prob[maxpos] / np.maximum((blur[maxpos] + shift), eps)
-    if inplace:
-        prob[:] = blur
-        blur = prob
-    np.add(blur, shift, out=blur)
-    np.multiply(blur, scale, out=blur)
+    import cv2
+    if prob.size == 0:
+        blur = prob.copy()
+    else:
+        sigma = 0.3 * ((k - 1) * 0.5 - 1) + 0.8  # opencv formula
+        blur = cv2.GaussianBlur(prob, (k, k), sigma)
+        # Shift and scale the intensities so the maximum and minimum
+        # pixel value in the blurred image match the original image
+        minpos = np.unravel_index(blur.argmin(), blur.shape)
+        maxpos = np.unravel_index(blur.argmax(), blur.shape)
+        shift = prob[minpos] - blur[minpos]
+        scale = prob[maxpos] / np.maximum((blur[maxpos] + shift), eps)
+        if inplace:
+            prob[:] = blur
+            blur = prob
+        np.add(blur, shift, out=blur)
+        np.multiply(blur, scale, out=blur)
     return blur
 
 
@@ -1767,6 +1846,7 @@ def _remove_translation(tf):
     TODO:
         - [ ] Is this possible in more general cases? E.g. projective transforms?
     """
+    import skimage
     if isinstance(tf, skimage.transform.AffineTransform):
         tf_notrans = skimage.transform.AffineTransform(
             scale=tf.scale, rotation=tf.rotation, shear=tf.shear)
