@@ -3,12 +3,17 @@ Data structures to represent and manipulate 2D Points
 """
 import numpy as np
 import ubelt as ub
-import skimage
 import kwarray
+import numbers
 import warnings
 from kwimage.structs import _generic
 
 
+__docstubs__ = """
+from kwimage._typing import SKImageGeometricTransform
+"""
+
+
 class _PointsWarpMixin:
 
     def _warp_imgaug(self, augmenter, input_dims, inplace=False):
@@ -33,7 +38,7 @@ class _PointsWarpMixin:
             >>> augmenter = imgaug.augmenters.Fliplr(p=1)
             >>> new = self._warp_imgaug(augmenter, input_dims)
 
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
             >>> plt = kwplot.autoplt()
             >>> kwplot.figure(fnum=1, doclf=True)
@@ -83,7 +88,7 @@ class _PointsWarpMixin:
         Generalized coordinate transform.
 
         Args:
-            transform (ArrayLike | Callable | kwimage.Affine | GeometricTransform | Augmenter):
+            transform (ArrayLike | Callable | kwimage.Affine | SKImageGeometricTransform | Augmenter):
                 scikit-image tranform, a 3x3 transformation matrix,
                 an imgaug Augmenter, or generic callable which transforms
                 an NxD ndarray.
@@ -97,6 +102,7 @@ class _PointsWarpMixin:
 
         Example:
             >>> from kwimage.structs.points import *  # NOQA
+            >>> import skimage
             >>> self = Points.random(10, rng=0)
             >>> transform = skimage.transform.AffineTransform(scale=(2, 2))
             >>> new = self.warp(transform)
@@ -108,12 +114,14 @@ class _PointsWarpMixin:
             >>> assert np.all(self.warp(np.eye(2)).xy == self.xy)
         """
         import kwimage
+        import skimage
+        from kwimage._typing import SKImageGeometricTransform
         new = self if inplace else self.__class__(self.data.copy(), self.meta)
         if transform is None:
             return new
 
         if not isinstance(transform, (np.ndarray,
-                                      skimage.transform._geometric.GeometricTransform,
+                                      SKImageGeometricTransform,
                                       kwimage.Affine)):
             try:
                 import imgaug
@@ -157,6 +165,7 @@ class _PointsWarpMixin:
             >>> new = self.scale(10)
             >>> assert new.xy.max() <= 10
         """
+        import skimage
         new = self if inplace else self.__class__(self.data.copy(), self.meta)
         new.data['xy'] = new.data['xy'].scale(factor, output_dims=output_dims,
                                               inplace=inplace)
@@ -184,6 +193,7 @@ class _PointsWarpMixin:
             >>> assert new.xy.min() >= 10
             >>> assert new.xy.max() <= 11
         """
+        import skimage
         new = self if inplace else self.__class__(self.data.copy(), self.meta)
         new.data['xy'] = new.data['xy'].translate(offset, output_dims, inplace)
         if 'tf_data_to_img' in new.meta:
@@ -240,7 +250,7 @@ class Points(_generic.Spatial, _PointsWarpMixin):
                     'Unknown kwargs: {}'.format(sorted(kwargs.keys())))
 
             if 'xy' in data:
-                if isinstance(data['xy'], _generic.ARRAY_TYPES):
+                if _generic.isinstance_arraytypes(data['xy']):
                     import kwimage
                     data['xy'] = kwimage.Coords(data['xy'])
 
@@ -354,17 +364,38 @@ class Points(_generic.Spatial, _PointsWarpMixin):
 
     def draw_on(self, image=None, color='white', radius=None, copy=False):
         """
+
+        Args:
+            image (ndarray): image to draw points on.
+
+            color (str | Any | List[Any]):
+                one color for all boxes or a list of colors for each box
+                Can be any type accepted by kwimage.Color.coerce.
+                Extended types: str | ColorLike | List[ColorLike]
+
+            radius (None | int):
+                if an integer, an circle is drawn at each xy point with this
+                radius.
+                if None, attempts to fill a single point with subpixel accuracy,
+                which generally means 4 pixels will be given some weight.
+                Note: color can only be a single value for all points in this
+                case.
+
+            copy (bool): if True, force a copy of the image, otherwise
+                try to draw inplace (may not work depending on dtype).
+
         CommandLine:
             xdoctest -m ~/code/kwimage/kwimage/structs/points.py Points.draw_on --show
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:cv2)
             >>> from kwimage.structs.points import *  # NOQA
             >>> s = 128
             >>> image = np.zeros((s, s))
             >>> self = Points.random(10).scale(s)
             >>> image = self.draw_on(image)
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
             >>> kwplot.figure(fnum=1, doclf=True)
             >>> kwplot.autompl()
@@ -373,26 +404,27 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             >>> kwplot.show_if_requested()
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> from kwimage.structs.points import *  # NOQA
             >>> s = 128
             >>> image = np.zeros((s, s))
             >>> self = Points.random(10).scale(s)
             >>> image = self.draw_on(image, radius=3, color='distinct')
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
             >>> kwplot.figure(fnum=1, doclf=True)
             >>> kwplot.autompl()
             >>> kwplot.imshow(image)
-            >>> self.draw(radius=3, alpha=.5, color='classes')
+            >>> #self.draw(radius=3, alpha=.5, color='classes')
             >>> kwplot.show_if_requested()
 
         Example:
+            >>> # xdoctest: +REQUIRES(module:cv2)
             >>> import kwimage
             >>> s = 32
             >>> self = kwimage.Points.random(10).scale(s)
-            >>> color = 'blue'
-            >>> # Test drawong on all channel + dtype combinations
+            >>> color = 'kitware_green'
+            >>> # Test drawing on all channel + dtype combinations
             >>> im3 = np.zeros((s, s, 3), dtype=np.float32)
             >>> im_chans = {
             >>>     'im3': im3,
@@ -407,45 +439,102 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             >>> for k, v in inputs.items():
             >>>     im, kw = v
             >>>     outputs[k] = self.draw_on(im, color=color, **kw)
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
             >>> kwplot.figure(fnum=2, doclf=True)
-            >>> kwplot.autompl()
-            >>> pnum_ = kwplot.PlotNums(nCols=2, nRows=len(inputs))
+            >>> plt = kwplot.autoplt()
+            >>> pnum_ = kwplot.PlotNums(nRows=2, nSubplots=len(inputs))
             >>> for k in inputs.keys():
-            >>>     kwplot.imshow(inputs[k][0], fnum=2, pnum=pnum_(), title=k)
             >>>     kwplot.imshow(outputs[k], fnum=2, pnum=pnum_(), title=k)
+            >>> plt.gcf().suptitle('Test draw points on channel + dtype combos')
             >>> kwplot.show_if_requested()
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> from kwimage.structs.points import *  # NOQA
             >>> self = Points.random(10).scale(32)
             >>> image = self.draw_on(radius=3, color='distinct')
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
             >>> kwplot.autompl()
             >>> kwplot.imshow(image)
             >>> kwplot.show_if_requested()
+
+        Example:
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> # Test cases where single and multiple colors are given
+            >>> # with radius=None and radius=scalar
+            >>> from kwimage.structs.points import *  # NOQA
+            >>> import kwimage
+            >>> self = kwimage.Points.random(10).scale(32)
+            >>> image1 = self.draw_on(radius=2, color='blue')
+            >>> image2 = self.draw_on(radius=None, color='blue')
+            >>> image3 = self.draw_on(radius=2, color='distinct')
+            >>> image4 = self.draw_on(radius=None, color='distinct')
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> import kwplot
+            >>> canvas = kwimage.stack_images_grid(
+            >>>     [image1, image2, image3, image4],
+            >>>     pad=3, bg_value=(1, 1, 1))
+            >>> kwplot.autompl()
+            >>> kwplot.imshow(canvas)
+            >>> kwplot.show_if_requested()
         """
         import kwimage
         if image is None:
             maxx, maxy = self.xy.max(axis=0)
-            maxx = int(np.ceil(maxx * 1.1))
-            maxy = int(np.ceil(maxy * 1.1))
+            maxx = int(np.ceil(maxx) + 1)
+            maxy = int(np.ceil(maxy) + 1)
+            image = np.zeros((maxy, maxx, 3), dtype=np.float32)
+        elif isinstance(image, tuple):
+            # I forgot what the standard is that we use here...
+            maxy, maxx = image
             image = np.zeros((maxx, maxy, 3), dtype=np.float32)
 
+        if 0 in image.shape[0:2]:
+            # Cannot draw on this image without width and height, return it as is
+            return image
+
         dtype_fixer = _generic._consistent_dtype_fixer(image)
 
+        single_color = False
+
+        if color == 'distinct':
+            colors = [kwimage.Color(c) for c in kwimage.Color.distinct(len(self))]
+        elif color == 'classes':
+            # TODO: read colors from categories if they exist
+            class_idxs = self.data['class_idxs']
+            _keys, _vals = kwarray.group_indices(class_idxs)
+            cls_colors = kwimage.Color.distinct(len(self.meta['classes']))
+            colors = list(ub.take(cls_colors, class_idxs))
+            colors = [kwimage.Color(c) for c in colors]
+        else:
+            num = len(self)
+            if isinstance(color, list) and not isinstance(color, numbers.Number):
+                # Passed list of color for each point
+                colors = [kwimage.Color(c) for c in color]
+            else:
+                # Passed a single color
+                single_color = True
+                colors = [kwimage.Color(color)] * num
+
         if radius is None:
-            if color == 'distinct':
-                raise NotImplementedError
             image = kwimage.atleast_3channels(image)
             image = kwimage.ensure_float01(image, copy=copy)
             # value = kwimage.Color(color).as01()
-            value = kwimage.Color(color)._forimage(image)
-            image = self.data['xy'].fill(
-                image, value, coord_axes=[1, 0], interp='bilinear')
+            if single_color:
+                color_value = np.array(colors[0].forimage(image))
+                image = self.data['xy'].fill(
+                    image, color_value, coord_axes=[1, 0], interp='bilinear')
+            else:
+                # Need to loop when thare are multiple colors
+                color_values = [np.array(kwimage.Color(c).forimage(image))
+                                for c in colors]
+                xy_pts = self.data['xy'].data.reshape(-1, 2)
+                for xy, color_ in zip(xy_pts, color_values):
+                    image = kwimage.subpixel_setvalue(
+                        image, xy[None, :], color_, coord_axes=[1, 0],
+                        interp='bilinear')
         else:
             import cv2
             image = kwimage.atleast_3channels(image, copy=copy)
@@ -454,26 +543,9 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             image = np.ascontiguousarray(image)
 
             xy_pts = self.data['xy'].data.reshape(-1, 2)
+            color_values = [kwimage.Color(c).forimage(image) for c in colors]
 
-            if color == 'distinct':
-                colors = kwimage.Color.distinct(len(xy_pts))
-            elif color == 'classes':
-                # TODO: read colors from categories if they exist
-                class_idxs = self.data['class_idxs']
-                _keys, _vals = kwarray.group_indices(class_idxs)
-                cls_colors = kwimage.Color.distinct(len(self.meta['classes']))
-                colors = list(ub.take(cls_colors, class_idxs))
-                colors = [kwimage.Color(c)._forimage(image) for c in colors]
-                # if image.dtype.kind == 'f':
-                #     colors = [kwimage.Color(c).as01() for c in colors]
-                # else:
-                #     colors = [kwimage.Color(c).as255() for c in colors]
-            else:
-                value = kwimage.Color(color)._forimage(image)
-                colors = [value] * len(xy_pts)
-                # image = kwimage.ensure_float01(image)
-
-            for xy, color_ in zip(xy_pts, colors):
+            for xy, color_ in zip(xy_pts, color_values):
                 # center = tuple(map(int, xy.tolist()))
                 center = tuple(xy.tolist())
                 axes = (radius / 2, radius / 2)
@@ -488,17 +560,18 @@ class Points(_generic.Spatial, _PointsWarpMixin):
         image = dtype_fixer(image, copy=False)
         return image
 
-    def draw(self, color='blue', ax=None, alpha=None, radius=1, **kwargs):
+    def draw(self, color='blue', ax=None, alpha=None, radius=1, setlim=False, **kwargs):
         """
         TODO: can use kwplot.draw_points
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> from kwimage.structs.points import *  # NOQA
             >>> pts = Points.random(10)
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> import kwplot
+            >>> kwplot.figure(doclf=1)
             >>> pts.draw(radius=0.01)
-
             >>> from kwimage.structs.points import *  # NOQA
             >>> self = Points.random(10, classes=['a', 'b', 'c'])
             >>> self.draw(radius=0.01, color='classes')
@@ -509,6 +582,7 @@ class Points(_generic.Spatial, _PointsWarpMixin):
         if ax is None:
             ax = plt.gca()
         xy = self.data['xy'].data.reshape(-1, 2)
+        # kwplot.draw_points(color=color, class_idxs)
 
         # More grouped patches == more efficient runtime
         if alpha is None:
@@ -553,6 +627,8 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             if fc is not None:
                 pcolor = fc
 
+            print(f'circlekw={circlekw}')
+            print(f'pcolor={pcolor}')
             patches = [
                 mpl.patches.Circle((x, y), fc=pcolor, **circlekw)
                 for x, y in xy[idxs]
@@ -560,6 +636,14 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             col = mpl.collections.PatchCollection(patches, match_original=True)
             collections.append(col)
             ax.add_collection(col)
+
+        if setlim:
+            xmin = xy[:, 0].min()
+            xmax = xy[:, 0].max()
+            ymin = xy[:, 1].min()
+            ymax = xy[:, 1].max()
+            _generic._setlim(xmin, ymin, xmax, ymax, setlim=setlim, ax=ax)
+
         return collections
 
     def compress(self, flags, axis=0, inplace=False):
@@ -630,7 +714,7 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             be rectified.
 
         Args:
-            style (str): either orig, new, new-id, or new-name
+            style (str): either orig, new, new-id, new-name, or new-v2
 
         Returns:
             Dict: mscoco-like representation
@@ -641,12 +725,15 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             >>> orig = self._to_coco(style='orig')
             >>> print('orig = {!r}'.format(orig))
             >>> new_name = self._to_coco(style='new-name')
-            >>> print('new_name = {}'.format(ub.repr2(new_name, nl=-1)))
+            >>> print('new_name = {}'.format(ub.urepr(new_name, nl=-1)))
             >>> # xdoctest: +REQUIRES(module:kwcoco)
             >>> import kwcoco
             >>> self.meta['classes'] = kwcoco.CategoryTree.coerce(self.meta['classes'])
             >>> new_id = self._to_coco(style='new-id')
-            >>> print('new_id = {}'.format(ub.repr2(new_id, nl=-1)))
+            >>> print('new_id = {}'.format(ub.urepr(new_id, nl=-1)))
+            >>> # Test new-v2 style
+            >>> new_v2 = self._to_coco(style='new-v2')
+            >>> print('new_v2 = {}'.format(ub.urepr(new_v2, nl=-1)))
         """
         if self.xy.size == 0:
             return []
@@ -670,8 +757,28 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             # TODO: ensure these are in the right order for the classes
             flat_pts = np.hstack([self.xy, visible]).reshape(-1)
             return flat_pts.tolist()
+        elif style.startswith('new-v2'):
+            new_kpts_v2 = {}
+            x, y = self.data['xy'].data.T.tolist()
+            new_kpts_v2['x'] = x
+            new_kpts_v2['y'] = y
+            class_idxs = self.data.get('class_idxs', None)
+            if class_idxs is not None:
+                assert 'classes' in self.meta
+                classes = self.meta['classes']
+                try:
+                    _idx_to_id = classes.idx_to_id
+                except AttributeError:
+                    catnames = [classes[idx] for idx in class_idxs]
+                    new_kpts_v2['keypoint_category_name'] = catnames
+                else:
+                    keypoint_category_id = [_idx_to_id[idx] for idx in class_idxs]
+                    new_kpts_v2['keypoint_category_id'] = keypoint_category_id
+            visible = self.data.get('visible', None)
+            if visible is not None:
+                new_kpts_v2['visible'] = self.data['visible'].tolist()
+            return new_kpts_v2
         elif style.startswith('new'):
-
             if style == 'new-id':
                 use_id = True
             elif style == 'new-name':
@@ -682,12 +789,14 @@ class Points(_generic.Spatial, _PointsWarpMixin):
                 raise KeyError(style)
 
             new_kpts = []
+            _visible = self.data.get('visible', None)
+            _class_idxs = self.data.get('class_idxs', None)
             for i, xy in enumerate(self.data['xy'].data.tolist()):
                 kpdict = {'xy': xy}
-                if 'visible' in self.data:
-                    kpdict['visible'] = int(self.data['visible'][i])
-                if 'class_idxs' in self.data:
-                    cidx = self.data['class_idxs'][i]
+                if _visible is not None:
+                    kpdict['visible'] = int(_visible[i])
+                if _class_idxs is not None:
+                    cidx = _class_idxs[i]
                     if use_id:
                         cid = self.meta['classes'].idx_to_id[cidx]
                         kpdict['keypoint_category_id'] = int(cid)
@@ -699,18 +808,88 @@ class Points(_generic.Spatial, _PointsWarpMixin):
         else:
             raise KeyError(style)
 
+    def to_wkt(self):
+        """
+        Convert points to well known text. Loses any metadata information.
+
+        Returns:
+            str
+
+        Example:
+            >>> import kwimage
+            >>> self = kwimage.Points.random(3, rng=0)
+            >>> self.to_wkt()
+        """
+        import shapely
+        xy = self.data['xy'].data
+        geom = shapely.geometry.multipoint.MultiPoint(xy)
+        return geom
+
+    def to_shapely(self):
+        """
+        Convert points to shapely. Loses any metadata information.
+
+        Returns:
+            shapely.geometry.multipoint.MultiPoint:
+
+        Example:
+            >>> import kwimage
+            >>> self = kwimage.Points.random(3)
+            >>> geom = self.to_shapely()
+            >>> print(f'geom={geom}')
+        """
+        geom = self.data['xy'].to_shapely()
+        return geom
+
     @classmethod
-    def coerce(cls, data):
+    def from_shapely(Points, geom):
+        """
+        Create a Points object from shapely.
+
+        Args:
+            geom (shapely.geometry.multipoint.MultiPoint):
+                a shapely multipoint
+
+        Returns:
+            Self:
+
+        Example:
+            >>> import kwimage
+            >>> self = kwimage.Points.random(3)
+            >>> geom = self.to_shapely()
+            >>> new = kwimage.Points.from_shapely(geom)
+            >>> assert np.isclose(self.xy, new.xy).all()
+        """
+        import kwimage
+        data = {}
+        data['xy'] = kwimage.Coords.from_shapely(geom)
+        self = Points(data)
+        return self
+
+    @classmethod
+    def coerce(cls, data, classes=None):
         """
         Attempt to coerce data into a Points object
+
+        Args:
+            data (list | dict | Points): data to attempt to coerce into points
+            classes (list[str] | kwcoco.CategoryTree):
+                list of all keypoint category names if converting from a
+                coco representation.
+
+        Example:
+            >>> import kwimage
+            >>> data = np.random.rand(10, 2)
+            >>> pts = kwimage.Points.coerce(data)
         """
         if isinstance(data, cls):
             return data
         elif isinstance(data, (list, dict)):
             # TODO: determine if coco or geojson
-            return cls.from_coco(data)
-        elif isinstance(data, _generic.ARRAY_TYPES):
-            return cls(data)
+            return cls.from_coco(data, classes=classes)
+        elif _generic.isinstance_arraytypes(data):
+            return cls(xy=data)
+        # TODO: check if a shapely object
         else:
             raise TypeError(type(data))
 
@@ -728,7 +907,7 @@ class Points(_generic.Spatial, _PointsWarpMixin):
 
             class_idxs (list): only needed if using old style
 
-            classes (list | kwcoco.CategoryTree):
+            classes (None | list[str] | kwcoco.CategoryTree):
                 list of all keypoint category names
 
             warn (bool): if True raise warnings
@@ -769,12 +948,73 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             >>> ]
             >>> pts = Points.from_coco(coco_kpts, classes=classes)
             >>> assert pts.data['class_idxs'].tolist() == [2, 0]
+
+        Example:
+            >>> # xdoctest: +REQUIRES(module:kwcoco)
+            >>> from kwimage.structs.points import *  # NOQA
+            >>> import kwcoco
+            >>> coco_kpts = {
+            >>>     'x': [0, 1, 2, 3, 4, 5],
+            >>>     'y': [0, 1, 2, 3, 4, 5],
+            >>> }
+            >>> # Without classes
+            >>> pts1 = Points.from_coco(coco_kpts)
+            >>> print(f'pts1.data={pts1.data}')
+            >>> print(f'pts1.meta={pts1.meta}')
+            >>> # With classes
+            >>> classes = kwcoco.CategoryTree.from_coco([
+            >>>     {'name': 'mouth', 'id': 2}, {'name': 'left-hand', 'id': 3}, {'name': 'right-hand', 'id': 5}
+            >>> ])
+            >>> coco_kpts['keypoint_category_id'] = [5, 2, 3, 5, 2]
+            >>> pts2 = Points.from_coco(coco_kpts, classes=classes)
+            >>> print(f'pts2 = {ub.urepr(pts2, nl=1)}')
+            >>> print(f'pts2.data={pts2.data}')
+            >>> print(f'pts2.meta={pts2.meta}')
+            >>> assert pts2.data['class_idxs'].tolist() == [2, 0, 1, 2, 0]
         """
         if coco_kpts is None:
             return None
 
-        if len(coco_kpts) and isinstance(ub.peek(coco_kpts), dict):
-            # new style
+        if isinstance(coco_kpts, dict) and 'x' in coco_kpts:
+            # new style (v2) which is column based and uses less memory
+            xs = coco_kpts['x']
+            ys = coco_kpts['y']
+            class_idxs = None
+            visible = None
+            if len(xs) != len(ys):
+                raise ValueError(
+                    'new-v2 column-style coco keypoints must have '
+                    f'corresponding length lists. Got: {len(xs)}, {len(ys)}'
+                )
+
+            visible = coco_kpts.get('visible', None)
+            if visible is None:
+                visible = np.array([2] * len(xs))
+            else:
+                visible = np.array(visible)
+
+            keypoint_category_id = coco_kpts.get('keypoint_category_id', None)
+            if keypoint_category_id is not None:
+                if classes is None:
+                    class_idxs = None
+                    # raise Exception('classes should be specified for new-style-v2')
+                else:
+                    try:
+                        class_idxs = [classes.id_to_idx[cid]
+                                      for cid in keypoint_category_id]
+                    except AttributeError:
+                        raise TypeError('classes needs to be a kwcoco.CategoryTree to parse keypoint_category_id')
+                    else:
+                        class_idxs = np.array(class_idxs)
+
+            xs = np.array(xs)
+            ys = np.array(ys)
+            xy = np.stack([xs, ys], axis=1)
+            self = cls(xy=xy, visible=visible, class_idxs=class_idxs,
+                       classes=classes)
+
+        elif len(coco_kpts) and isinstance(ub.peek(coco_kpts), dict):
+            # new style (v1)
             xy = []
             visible = []
             cidx_list = []
@@ -784,6 +1024,8 @@ class Points(_generic.Spatial, _PointsWarpMixin):
                     warnings.warn('class_idxs should not be specified for new-style')
                 class_idxs = None
 
+            drop_cidx = False
+
             # raise NotImplementedError(
             #     '''
             #     Needs to have extra information available to map
@@ -840,7 +1082,9 @@ class Points(_generic.Spatial, _PointsWarpMixin):
                 else:
                     if 'keypoint_category_id' in kpdict or 'keypoint_category' in kpdict:
                         # warnings.warn('classes should be specified for new-style')
-                        raise Exception('classes should be specified for new-style')
+                        # raise Exception('classes should be specified for new-style')
+                        cidx_list.append(None)
+                        drop_cidx = True
 
                 xy.append(kpdict['xy'])
                 visible.append(kpdict.get('visible', 2))
@@ -850,7 +1094,10 @@ class Points(_generic.Spatial, _PointsWarpMixin):
             else:
                 cidx_list = None
 
-            cidx_list = np.array(cidx_list)
+            if drop_cidx:
+                cidx_list = None
+            else:
+                cidx_list = np.array(cidx_list)
 
             xy = np.array(xy)
             visible = np.array(visible)
