@@ -1,4 +1,8 @@
 """
+This file originally contained tools for warping with respect to tensors, but
+has been expanded to include the generic :func:`warp_affine` function that may
+use multiple backends.
+
 TODO:
     - [ ] Replace internal padded slice with kwarray.padded_slice
 """
@@ -7,18 +11,9 @@ import numpy as np
 import kwarray
 
 try:
-    from packaging.version import parse as LooseVersion
-except ImportError:
-    from distutils.version import LooseVersion
-
-try:
-    import torch
-    import torch.nn.functional as F
-    TORCH_GRID_SAMPLE_HAS_ALIGN = LooseVersion(torch.__version__) >= LooseVersion('1.3.0')
+    from line_profiler import profile
 except Exception:
-    torch = None
-    F = None
-    TORCH_GRID_SAMPLE_HAS_ALIGN = None
+    profile = ub.identity
 
 
 def _coordinate_grid(dims, align_corners=False):
@@ -37,7 +32,7 @@ def _coordinate_grid(dims, align_corners=False):
             with shape=(3, *DIMS)
 
     References:
-        https://github.com/ClementPinard/SfmLearner-Pytorch/blob/master/inverse_warp.py
+        .. [InvWarp] https://github.com/ClementPinard/SfmLearner-Pytorch/blob/master/inverse_warp.py
 
     Example:
         >>> # xdoctest: +IGNORE_WHITESPACE
@@ -59,6 +54,7 @@ def _coordinate_grid(dims, align_corners=False):
                  [1., 1.]]])
 
     """
+    import torch
     if align_corners:
         def _corner_grid(d):
             return torch.linspace(0, d, d)
@@ -87,15 +83,6 @@ def _coordinate_grid(dims, align_corners=False):
     return pixel_coords
 
 
-def warp_image(inputs, mat, **kw):
-    raise AssertionError('do not use')
-    # import kwarray
-    # _impl = kwarray.ArrayAPI.coerce(inputs)
-    # inputs = kwarray.atleast_nd(inputs, 3)
-    # tensor = inputs.transpose(2, 0, 1)
-    # return warp_tensor(tensor, mat, **kw)
-
-
 def warp_tensor(inputs, mat, output_dims, mode='bilinear',
                 padding_mode='zeros', isinv=False, ishomog=None,
                 align_corners=False, new_mode=False):
@@ -154,6 +141,8 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
             rotation or shear are non-zero. I'm not sure what the cause is.
             It may just be floating point issues, but Im' not sure.
 
+        See issues in [TorchAffineTransform]_ and [TorchIssue15386]_.
+
     TODO:
         - [ ] FIXME: see example in Mask.scale where this algo breaks when the matrix is `2x3`
         - [ ] Make this algo work when matrix ix 2x2
@@ -165,7 +154,9 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
     Example:
         >>> # Create a relatively simple affine matrix
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> import skimage
+        >>> import torch
         >>> mat = torch.FloatTensor(skimage.transform.AffineTransform(
         >>>     translation=[1, -1], scale=[.532, 2],
         >>>     rotation=0, shear=0,
@@ -176,21 +167,23 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>> output_dims = (11, 7)
         >>> # Warp with our code
         >>> result1 = warp_tensor(inputs, mat, output_dims=output_dims, align_corners=0)
-        >>> print('result1 =\n{}'.format(ub.repr2(result1.cpu().numpy()[0, 0], precision=2)))
+        >>> print('result1 =\n{}'.format(ub.urepr(result1.cpu().numpy()[0, 0], precision=2)))
         >>> # Warp with opencv
         >>> import cv2
         >>> cv2_M = mat.cpu().numpy()[0:2]
         >>> src = inputs[0, 0].cpu().numpy()
         >>> dsize = tuple(output_dims[::-1])
         >>> result2 = cv2.warpAffine(src, cv2_M, dsize=dsize, flags=cv2.INTER_LINEAR)
-        >>> print('result2 =\n{}'.format(ub.repr2(result2, precision=2)))
+        >>> print('result2 =\n{}'.format(ub.urepr(result2, precision=2)))
         >>> # Ensure the results are the same (up to floating point errors)
         >>> assert np.all(np.isclose(result1[0, 0].cpu().numpy(), result2, atol=1e-2, rtol=1e-2))
 
     Example:
         >>> # Create a relatively simple affine matrix
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> import skimage
+        >>> import torch
         >>> mat = torch.FloatTensor(skimage.transform.AffineTransform(
         >>>     rotation=0.01, shear=0.1).params)
         >>> # Create inputs and an output dimension
@@ -199,7 +192,7 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>> output_dims = (11, 7)
         >>> # Warp with our code
         >>> result1 = warp_tensor(inputs, mat, output_dims=output_dims)
-        >>> print('result1 =\n{}'.format(ub.repr2(result1.cpu().numpy()[0, 0], precision=2, supress_small=True)))
+        >>> print('result1 =\n{}'.format(ub.urepr(result1.cpu().numpy()[0, 0], precision=2, supress_small=True)))
         >>> print('result1.shape = {}'.format(result1.shape))
         >>> # Warp with opencv
         >>> import cv2
@@ -207,7 +200,7 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>> src = inputs[0, 0].cpu().numpy()
         >>> dsize = tuple(output_dims[::-1])
         >>> result2 = cv2.warpAffine(src, cv2_M, dsize=dsize, flags=cv2.INTER_LINEAR)
-        >>> print('result2 =\n{}'.format(ub.repr2(result2, precision=2)))
+        >>> print('result2 =\n{}'.format(ub.urepr(result2, precision=2)))
         >>> print('result2.shape = {}'.format(result2.shape))
         >>> # Ensure the results are the same (up to floating point errors)
         >>> # NOTE: The floating point errors seem to be significant for rotation / shear
@@ -216,7 +209,9 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
     Example:
         >>> # Create a random affine matrix
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> import skimage
+        >>> import torch
         >>> rng = np.random.RandomState(0)
         >>> mat = torch.FloatTensor(skimage.transform.AffineTransform(
         >>>     translation=rng.randn(2), scale=1 + rng.randn(2),
@@ -228,14 +223,14 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>> output_dims = (3, 11)
         >>> # Warp with our code
         >>> result1 = warp_tensor(inputs, mat, output_dims=output_dims, align_corners=0)
-        >>> print('result1 =\n{}'.format(ub.repr2(result1.cpu().numpy()[0, 0], precision=2)))
+        >>> print('result1 =\n{}'.format(ub.urepr(result1.cpu().numpy()[0, 0], precision=2)))
         >>> # Warp with opencv
         >>> import cv2
         >>> cv2_M = mat.cpu().numpy()[0:2]
         >>> src = inputs[0, 0].cpu().numpy()
         >>> dsize = tuple(output_dims[::-1])
         >>> result2 = cv2.warpAffine(src, cv2_M, dsize=dsize, flags=cv2.INTER_LINEAR)
-        >>> print('result2 =\n{}'.format(ub.repr2(result2, precision=2)))
+        >>> print('result2 =\n{}'.format(ub.urepr(result2, precision=2)))
         >>> # Ensure the results are the same (up to floating point errors)
         >>> # NOTE: The errors seem to be significant for rotation / shear
         >>> assert np.all(np.isclose(result1[0, 0].cpu().numpy(), result2, atol=1, rtol=1e-2))
@@ -243,18 +238,20 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
     Example:
         >>> # Test 3D warping with identity
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> mat = torch.eye(4)
         >>> input_dims = [2, 3, 3]
         >>> output_dims = (2, 3, 3)
         >>> input_shape = [1, 1] + input_dims
         >>> inputs = torch.arange(int(np.prod(input_shape))).reshape(*input_shape).float()
         >>> result = warp_tensor(inputs, mat, output_dims=output_dims)
-        >>> print('result =\n{}'.format(ub.repr2(result.cpu().numpy()[0, 0], precision=2)))
+        >>> print('result =\n{}'.format(ub.urepr(result.cpu().numpy()[0, 0], precision=2)))
         >>> assert torch.all(inputs == result)
 
     Example:
         >>> # Test 3D warping with scaling
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> mat = torch.FloatTensor([
         >>>     [0.8,   0,   0, 0],
         >>>     [  0, 1.0,   0, 0],
@@ -266,7 +263,7 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>> input_shape = [1, 1] + input_dims
         >>> inputs = torch.arange(int(np.prod(input_shape))).reshape(*input_shape).float()
         >>> result = warp_tensor(inputs, mat, output_dims=output_dims, align_corners=0)
-        >>> print('result =\n{}'.format(ub.repr2(result.cpu().numpy()[0, 0], precision=2)))
+        >>> print('result =\n{}'.format(ub.urepr(result.cpu().numpy()[0, 0], precision=2)))
         result =
         np.array([[[ 0.  ,  1.25,  1.  ],
                    [ 3.  ,  4.25,  2.5 ],
@@ -278,6 +275,7 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
 
     Example:
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> mat = torch.eye(3)
         >>> input_dims = [5, 7]
         >>> output_dims = (11, 7)
@@ -285,11 +283,12 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>>      input_shape = [2] * n_prefix_dims + input_dims
         >>>      inputs = torch.arange(int(np.prod(input_shape))).reshape(*input_shape).float()
         >>>      result = warp_tensor(inputs, mat, output_dims=output_dims)
-        >>>      #print('result =\n{}'.format(ub.repr2(result.cpu().numpy(), precision=2)))
+        >>>      #print('result =\n{}'.format(ub.urepr(result.cpu().numpy(), precision=2)))
         >>>      print(result.shape)
 
     Example:
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> mat = torch.eye(4)
         >>> input_dims = [5, 5, 5]
         >>> output_dims = (6, 6, 6)
@@ -297,19 +296,20 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>>      input_shape = [2] * n_prefix_dims + input_dims
         >>>      inputs = torch.arange(int(np.prod(input_shape))).reshape(*input_shape).float()
         >>>      result = warp_tensor(inputs, mat, output_dims=output_dims)
-        >>>      #print('result =\n{}'.format(ub.repr2(result.cpu().numpy(), precision=2)))
+        >>>      #print('result =\n{}'.format(ub.urepr(result.cpu().numpy(), precision=2)))
         >>>      print(result.shape)
 
     Ignore:
         import xdev
         globals().update(xdev.get_func_kwargs(warp_tensor))
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> import cv2
         >>> inputs = torch.arange(9).view(1, 1, 3, 3).float() + 2
         >>> input_dims = inputs.shape[2:]
         >>> #output_dims = (6, 6)
         >>> def fmt(a):
-        >>>     return ub.repr2(a.numpy(), precision=2)
+        >>>     return ub.urepr(a.numpy(), precision=2)
         >>> s = 2.5
         >>> output_dims = tuple(np.round((np.array(input_dims) * s)).astype(int).tolist())
         >>> mat = torch.FloatTensor([[s, 0, 0], [0, s, 0], [0, 0, 1]])
@@ -332,15 +332,16 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
         >>> print('## interpolate, align_corners=False')
         >>> print(fmt(F.interpolate(inputs, output_dims, mode='bilinear', align_corners=False)))
         >>> print('## interpolate (scale), align_corners=False')
-        >>> print(ub.repr2(F.interpolate(inputs, scale_factor=s, mode='bilinear', align_corners=False).numpy(), precision=2))
+        >>> print(ub.urepr(F.interpolate(inputs, scale_factor=s, mode='bilinear', align_corners=False).numpy(), precision=2))
         >>> cv2_M = mat.cpu().numpy()[0:2]
         >>> src = inputs[0, 0].cpu().numpy()
         >>> dsize = tuple(output_dims[::-1])
         >>> print('\nOpen CV warp Result')
         >>> result2 = (cv2.warpAffine(src, cv2_M, dsize=dsize, flags=cv2.INTER_LINEAR))
-        >>> print('result2 =\n{}'.format(ub.repr2(result2, precision=2)))
+        >>> print('result2 =\n{}'.format(ub.urepr(result2, precision=2)))
     """
-
+    import torch
+    import torch.nn.functional as F
     if mode == 'linear':
         mode = 'bilinear'
 
@@ -413,11 +414,6 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
     # X = ndims + 1 if ishomog else ndims
     X = ndims + 1
 
-    if not TORCH_GRID_SAMPLE_HAS_ALIGN:
-        import warnings
-        warnings.warn('cannot use new mode in warp_tensor when torch < 1.3')
-        new_mode = False
-
     # NOTE: grid_sample in torch<1.3 does not support align_corners=False correctly
     unwarped_coords = _coordinate_grid(output_dims, align_corners=align_corners)     # [X, *DIMS]
     unwarped_coords = unwarped_coords.to(device)
@@ -508,7 +504,7 @@ def warp_tensor(inputs, mat, output_dims, mode='bilinear',
     # TODO: pass align_corners when supported in torch 1.3
     # Note: enabling this breaks tests and backwards compat, so
     # verify there are no problems before enabling this.
-    if new_mode and TORCH_GRID_SAMPLE_HAS_ALIGN:
+    if new_mode:
         # the new grid sample allows you to set align_corners, but I don't
         # remember if the previous logic depends on the old behavior.
         outputs_ = F.grid_sample(inputs_, grid_coords, mode=mode,
@@ -612,7 +608,7 @@ def subpixel_set(dst, src, index, interp_axes=None):
         >>> src = np.ones(2)
         >>> index = [slice(1.5, 3.5)]
         >>> kwimage.util_warp.subpixel_set(dst, src, index)
-        >>> print(ub.repr2(dst, precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst, precision=2, with_dtype=0))
         np.array([0.1, 0.5, 1. , 0.5, 0.1])
     """
     aligned_src, aligned_index = subpixel_align(dst, src, index, interp_axes)
@@ -649,7 +645,7 @@ def subpixel_accum(dst, src, index, interp_axes=None):
         index (Tuple[slice]): subpixel slice into dst that corresponds with src
         interp_axes (tuple): specify which axes should be spatially interpolated
 
-    Ascii:
+    TextArt:
         Inputs:
             +---+---+---+---+---+  dst.shape = (5,)
                   +---+---+        src.shape = (2,)
@@ -669,7 +665,7 @@ def subpixel_accum(dst, src, index, interp_axes=None):
         >>> src = np.ones(2)
         >>> index = [slice(1.5, 3.5)]
         >>> subpixel_accum(dst, src, index)
-        >>> print(ub.repr2(dst, precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst, precision=2, with_dtype=0))
         np.array([0. , 0.5, 1. , 0.5, 0. ])
 
     Example:
@@ -677,7 +673,7 @@ def subpixel_accum(dst, src, index, interp_axes=None):
         >>> src = np.ones((3, 3))
         >>> index = (slice(1.5, 4.5), slice(1, 4))
         >>> subpixel_accum(dst, src, index)
-        >>> print(ub.repr2(dst, precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst, precision=2, with_dtype=0))
         np.array([[0. , 0. , 0. , 0. , 0. , 0. ],
                   [0. , 0.5, 0.5, 0.5, 0. , 0. ],
                   [0. , 1. , 1. , 1. , 0. , 0. ],
@@ -685,11 +681,12 @@ def subpixel_accum(dst, src, index, interp_axes=None):
                   [0. , 0.5, 0.5, 0.5, 0. , 0. ],
                   [0. , 0. , 0. , 0. , 0. , 0. ]])
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> dst = torch.zeros((1, 3, 6, 6))
         >>> src = torch.ones((1, 3, 3, 3))
         >>> index = (slice(None), slice(None), slice(1.5, 4.5), slice(1.25, 4.25))
         >>> subpixel_accum(dst, src, index)
-        >>> print(ub.repr2(dst.numpy()[0, 0], precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst.numpy()[0, 0], precision=2, with_dtype=0))
         np.array([[0.  , 0.  , 0.  , 0.  , 0.  , 0.  ],
                   [0.  , 0.38, 0.5 , 0.5 , 0.12, 0.  ],
                   [0.  , 0.75, 1.  , 1.  , 0.25, 0.  ],
@@ -761,16 +758,17 @@ def subpixel_maximum(dst, src, index, interp_axes=None):
         >>> src = np.array([2.0, 2.0])
         >>> index = [slice(1.6, 3.6)]
         >>> subpixel_maximum(dst, src, index)
-        >>> print(ub.repr2(dst, precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst, precision=2, with_dtype=0))
         np.array([0. , 1. , 2. , 1.2, 0. ])
 
     Example:
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> dst = torch.zeros((1, 3, 5, 5)) + .5
         >>> src = torch.ones((1, 3, 3, 3))
         >>> index = (slice(None), slice(None), slice(1.4, 4.4), slice(1.25, 4.25))
         >>> subpixel_maximum(dst, src, index)
-        >>> print(ub.repr2(dst.numpy()[0, 0], precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst.numpy()[0, 0], precision=2, with_dtype=0))
         np.array([[0.5 , 0.5 , 0.5 , 0.5 , 0.5 ],
                   [0.5 , 0.5 , 0.6 , 0.6 , 0.5 ],
                   [0.5 , 0.75, 1.  , 1.  , 0.5 ],
@@ -799,16 +797,17 @@ def subpixel_minimum(dst, src, index, interp_axes=None):
         >>> src = np.array([2.0, 2.0])
         >>> index = [slice(1.6, 3.6)]
         >>> subpixel_minimum(dst, src, index)
-        >>> print(ub.repr2(dst, precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst, precision=2, with_dtype=0))
         np.array([0. , 0.8, 1. , 1. , 0. ])
 
     Example:
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> dst = torch.zeros((1, 3, 5, 5)) + .5
         >>> src = torch.ones((1, 3, 3, 3))
         >>> index = (slice(None), slice(None), slice(1.4, 4.4), slice(1.25, 4.25))
         >>> subpixel_minimum(dst, src, index)
-        >>> print(ub.repr2(dst.numpy()[0, 0], precision=2, with_dtype=0))
+        >>> print(ub.urepr(dst.numpy()[0, 0], precision=2, with_dtype=0))
         np.array([[0.5 , 0.5 , 0.5 , 0.5 , 0.5 ],
                   [0.5 , 0.45, 0.5 , 0.5 , 0.15],
                   [0.5 , 0.5 , 0.5 , 0.5 , 0.25],
@@ -928,6 +927,7 @@ def subpixel_translate(inputs, shift, interp_axes=None, output_shape=None):
 
     Example:
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> inputs = torch.arange(9).view(1, 1, 3, 3).float()
         >>> print(inputs.long())
         tensor([[[[0, 1, 2],
@@ -1107,19 +1107,19 @@ def _padded_slice(data, in_slice, ndim=None, pad_slice=None,
         >>> in_slice = [slice(-2, 7)]
 
         >>> data_sliced, st_dims = _padded_slice(data, in_slice)
-        >>> print(ub.repr2(data_sliced, with_dtype=False))
+        >>> print(ub.urepr(data_sliced, with_dtype=False))
         >>> print(st_dims)
         np.array([0, 0, 0, 1, 2, 3, 4, 0, 0])
         [(-2, 7)]
 
         >>> data_sliced, st_dims = _padded_slice(data, in_slice, pad_slice=(3, 3))
-        >>> print(ub.repr2(data_sliced, with_dtype=False))
+        >>> print(ub.urepr(data_sliced, with_dtype=False))
         >>> print(st_dims)
         np.array([0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 0, 0, 0, 0, 0])
         [(-5, 10)]
 
         >>> data_sliced, st_dims = _padded_slice(data, slice(3, 4), pad_slice=[(1, 0)])
-        >>> print(ub.repr2(data_sliced, with_dtype=False))
+        >>> print(ub.urepr(data_sliced, with_dtype=False))
         >>> print(st_dims)
         np.array([2, 3])
         [(2, 4)]
@@ -1260,11 +1260,13 @@ def _warp_tensor_cv2(inputs, mat, output_dims, mode='linear', ishomog=None):
 
     Benchmark:
         >>> # xdoctest: +REQUIRES(module:torch)
-        >>> from kwimage.util.util_warp import *
-        >>> from kwimage.util.util_warp import _warp_tensor_cv2
-        >>> from kwimage.util.util_warp import warp_tensor
+        >>> from kwimage.util_warp import *
+        >>> from kwimage.util_warp import _warp_tensor_cv2
+        >>> from kwimage.util_warp import warp_tensor
+        >>> import timerit
+        >>> import torch
         >>> import numpy as np
-        >>> ti = ub.Timerit(10, bestof=3, verbose=2, unit='ms')
+        >>> ti = timerit.Timerit(10, bestof=3, verbose=2, unit='ms')
         >>> mode = 'linear'
         >>> rng = np.random.RandomState(0)
         >>> inputs = torch.Tensor(rng.rand(16, 10, 32, 32)).to('cpu')
@@ -1292,11 +1294,11 @@ def _warp_tensor_cv2(inputs, mat, output_dims, mode='linear', ishomog=None):
         >>>     b = kwarray.ArrayAPI.numpy(results[k2])
         >>>     diff = np.abs(a - b)
         >>>     diff_stats = kwarray.stats_dict(diff, n_extreme=1, extreme=1)
-        >>>     print('{} - {}: {}'.format(k1, k2, ub.repr2(diff_stats, nl=0, precision=4)))
+        >>>     print('{} - {}: {}'.format(k1, k2, ub.urepr(diff_stats, nl=0, precision=4)))
         >>> # xdoctest: +REQUIRES(--show)
         >>> import kwplot
         >>> kwplot.autompl()
-        >>> kwplot.imshow(results['warp_tensor(torch)'][0, 0], fnum=1, pnum=(1, 2, 1), title='torch')
+        >>> kwplot.imshow(results['warp_tensor(torch)'][0, 0].numpy(), fnum=1, pnum=(1, 2, 1), title='torch')
         >>> kwplot.imshow(results['warp_tensor(cv2)'][0, 0], fnum=1, pnum=(1, 2, 2), title='cv2')
     """
     import cv2
@@ -1341,6 +1343,7 @@ def _warp_tensor_cv2(inputs, mat, output_dims, mode='linear', ishomog=None):
     return outputs
 
 
+@profile
 def warp_points(matrix, pts, homog_mode='divide'):
     """
     Warp ND points / coordinates using a transformation matrix.
@@ -1377,6 +1380,7 @@ def warp_points(matrix, pts, homog_mode='divide'):
         >>> warp_points(matrix, pts)
         >>> # --- with torch
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> pts = torch.Tensor(pts)
         >>> matrix = torch.Tensor(matrix)
         >>> warp_points(matrix, pts)
@@ -1388,6 +1392,7 @@ def warp_points(matrix, pts, homog_mode='divide'):
         >>> matrix = np.diag([2, 3, 1])
         >>> ra = warp_points(matrix, pts)
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> rb = warp_points(torch.Tensor(matrix), torch.Tensor(pts))
         >>> assert np.allclose(ra, rb.numpy())
 
@@ -1400,6 +1405,7 @@ def warp_points(matrix, pts, homog_mode='divide'):
         >>> matrix = rng.rand(3, 3)
         >>> ra33 = warp_points(matrix, pts)
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> rb33 = warp_points(torch.Tensor(matrix), torch.Tensor(pts))
         >>> assert np.allclose(ra33, rb33.numpy())
         >>> # Test opencv style affine matrices
@@ -1419,12 +1425,14 @@ def warp_points(matrix, pts, homog_mode='divide'):
     D1, D2 = matrix.shape
 
     # Reshape points into a NxD, and transpose into a
-    pts_T = impl.T(impl.view(pts, (-1, D)))
+    _flat_pts = impl.view(pts, (-1, D))
+    pts_T = impl.T(_flat_pts)
 
     if D != D2:
         assert D + 1 == D2, 'only can have one homog coord'
         # Add homogenous coordinate
-        new_pts_T = impl.cat([pts_T, impl.ones_like(pts_T[0:1])], axis=0)
+        _ones = impl.ones_like(pts_T[0:1])
+        new_pts_T = impl.cat([pts_T, _ones], axis=0)
     else:
         # new_pts_T = impl.contiguous(pts_T)
         new_pts_T = pts_T
@@ -1542,6 +1550,7 @@ def subpixel_getvalue(img, pts, coord_axes=None, interp='bilinear',
         >>> subpixel_getvalue(img, pts, coord_axes=(1, 0))
         array([4. , 6. , 5.2])
         >>> # xdoctest: +REQUIRES(module:torch)
+        >>> import torch
         >>> img = torch.Tensor(img)
         >>> pts = torch.Tensor(pts)
         >>> subpixel_getvalue(img, pts)
@@ -1554,7 +1563,7 @@ def subpixel_getvalue(img, pts, coord_axes=None, interp='bilinear',
         tensor([4., 8., 7.])
 
     References:
-        stackoverflow.com/uestions/12729228/simple-binlin-interp-images-numpy
+        .. [SO12729228] stackoverflow.com/questions/12729228/simple-binlin-interp-images-numpy
 
     SeeAlso:
         cv2.getRectSubPix(image, patchSize, center[, patch[, patchType]])
@@ -1684,7 +1693,11 @@ def subpixel_setvalue(img, pts, value, coord_axes=None,
 
 def _bilinear_coords(ptsT, impl, img, coord_axes):
     i, j = coord_axes
-    height, width = img.shape[0:2]
+
+    r_extent = img.shape[i]
+    c_extent = img.shape[j]
+
+    # height, width = img.shape[0:2]
     ndims = len(img.shape)
 
     r, c = ptsT
@@ -1695,10 +1708,10 @@ def _bilinear_coords(ptsT, impl, img, coord_axes):
 
     # Make sure the values do not go past the boundary
     # Note: this is equivalent to bordermode=edge
-    c0 = impl.clip(c0, 0, width - 1, out=c0)
-    c1 = impl.clip(c1, 0, width - 1, out=c1)
-    r0 = impl.clip(r0, 0, height - 1, out=r0)
-    r1 = impl.clip(r1, 0, height - 1, out=r1)
+    c0 = impl.clip(c0, 0, c_extent - 1, out=c0)
+    c1 = impl.clip(c1, 0, c_extent - 1, out=c1)
+    r0 = impl.clip(r0, 0, r_extent - 1, out=r0)
+    r1 = impl.clip(r1, 0, r_extent - 1, out=r1)
 
     # Find bilinear weights
     alpha0 = (c - c0)
